@@ -6,10 +6,9 @@ from mobile_env.core.entities import UserEquipment
 
 
 class Arrival:
-    def __init__(self, ep_time: int,
-                 seed: int,
-                 reset_rng_episode: bool,
-                 **kwargs):
+    def __init__(
+        self, ep_time: int, seed: int, reset_rng_episode: bool, **kwargs
+    ):
         self.ep_time = ep_time
         self.seed = seed
         self.reset_rng_episode = reset_rng_episode
@@ -32,6 +31,7 @@ class Arrival:
 
 class NoDeparture(Arrival):
     """All UEs immediately request service and do not depart thereafter."""
+
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
 
