@@ -1,11 +1,12 @@
 from mobile_env.core.base import MComCore
 from mobile_env.core.entities import BaseStation, UserEquipment
+from mobile_env.core.util import deep_dict_merge
 
 
 class MComLarge(MComCore):
     def __init__(self, config={}):
         # set unspecified parameters to default configuration
-        config = {**self.default_config(), **config}
+        config = deep_dict_merge(self.default_config(), config)
 
         config.update({"width": 300, "height": 300})
         stations = [
@@ -31,8 +32,7 @@ class MComLarge(MComCore):
 
         num_ues = 30
         ues = [
-            UserEquipment(ue_id, **config["ue"])
-            for ue_id in range(num_ues)
+            UserEquipment(ue_id, **config["ue"]) for ue_id in range(num_ues)
         ]
 
         super().__init__(stations, ues, config)
