@@ -1,11 +1,12 @@
 from mobile_env.core.base import MComCore
 from mobile_env.core.entities import BaseStation, UserEquipment
+from mobile_env.core.util import deep_dict_merge
 
 
 class MComSmall(MComCore):
     def __init__(self, config={}):
         # set unspecified parameters to default configuration
-        config = {**self.default_config(), **config}
+        config = deep_dict_merge(self.default_config(), config)
 
         station_pos = [(110, 130), (65, 80), (120, 30)]
         stations = [
@@ -14,8 +15,7 @@ class MComSmall(MComCore):
         ]
         num_ues = 5
         ues = [
-            UserEquipment(ue_id, **config["ue"])
-            for ue_id in range(num_ues)
+            UserEquipment(ue_id, **config["ue"]) for ue_id in range(num_ues)
         ]
 
         super().__init__(stations, ues, config)
